<?php

/**
 *
 * @link              https://forumial.com
 * @since             1.0.0
 * @package           Forumial-SSO
 *
 * @wordpress-plugin
 * Plugin Name:       Forumial - Cloud Forum Platform - SSO
 * Plugin URI:        https://wordpress.org/plugins/forumial-sso
 * Description:       Forumial - Cloud Forum Platform. Easily create a cloud forum in just 1-minute. Free plan available.
 * Version:           1.0.0
 * Author:            Chris Nguyen
 * Author URI:        https://forumial.com
 */

// If this file is called directly, abort.
if ( ! defined( 'WPINC' ) ) {
	die;
}

define( 'FORUMIALSSO_VERSION', '1.0.0' );

define( 'FORUMIALSSO_HOMEPAGE', 'https://forumial.com/' );

function forumial_sso_installer() {
    require_once 'includes/installation.php';
}

register_activation_hook(__file__, 'forumial_sso_installer');

// Delete table when deactivate
function forumial_sso_remove_database() {
    global $wpdb;
    $table_name = $wpdb->prefix . 'forumial_sso_clients';
    $sql = "DROP TABLE IF EXISTS $table_name;";
    $wpdb->query($sql);
}

register_deactivation_hook( __FILE__, 'forumial_sso_remove_database' );

// Add settings link on plugin page
function forumial_sso_settings_link($links) {
    $settings_link = '<a href="admin.php?page=forumial-options">'.__('Settings', 'forumial-sso').'</a>';
    array_unshift($links, $settings_link);
    return $links;
}

$plugin = plugin_basename(__FILE__);
add_filter('plugin_action_links_'.$plugin, 'forumial_sso_settings_link');

function forumial_sso_random_hex($valLength = 32) {
    $result = '';
    $moduleLength = 40;   // we use sha1, so module is 40 chars
    $steps = ceil($valLength/$moduleLength);

    for( $i=0; $i<$steps; $i++ ) {
        $result .= sha1( uniqid() . md5( rand() . uniqid() ) );
    }

    return substr( $result, 0, $valLength );
}

function forumial_sso_register_settings() {
    add_option('forumial_clientid', substr(forumial_sso_random_hex(), 0, 32) );
    add_option('forumial_secret', 'fm_' . substr(forumial_sso_random_hex(), 0, 29) );
    register_setting('default', 'forumial_clientid');
    register_setting('default', 'forumial_secret');
}

add_action('admin_init', 'forumial_sso_register_settings');

function forumial_sso_register_options_page() {
    add_options_page('Forumial SSO', 'Forumial SSO', 'manage_options', 'forumial-options', 'forumial_sso_options_page');
}

add_action('admin_menu', 'forumial_sso_register_options_page');

function forumial_sso_options_page() {
    ?>
    <div class="wrap">
        <?php screen_icon(); ?>
        <h2><?= __('Forumial - Cloud Forum Platform SSO Settings', 'forumial-sso') ?></h2>

        <div class="wo-background">
            <form method="post" action="options.php"> 
                <?php settings_fields('default'); ?>

                <p>Create your Free Cloud Forum <a href="<?= FORUMIALSSO_HOMEPAGE ?>" target="_blank">here</a>.</p>
                <p>Note: The values below must match with values you entered in Forumial -> Admin -> Single Sign On -> Wordpress</p>
                <table class="form-table">
                    <tr valign="top">
                        <th scope="row"><label for="forumial_clientid"><?= __('OAuth Server:', 'forumial-sso') ?></label></th>
                        <td><code style="font-size:110%;"><?= home_url(); ?></code></td>
                    </tr>
                    <tr valign="top">
                        <th scope="row"><label for="forumial_clientid"><?= __('SSO Client ID:', 'forumial-sso') ?></label></th>
                        <td><input type="text" id="forumial_clientid" name="forumial_clientid" class="large-text" value="<?php echo get_option('forumial_clientid'); ?>" readonly="readonly" /></td>
                    </tr>
                    <tr valign="top">
                        <th scope="row"><label for="forumial_secret"><?= __('Secret:', 'forumial-sso') ?></label></th>
                        <td><input type="text" id="forumial_secret" name="forumial_secret" class="large-text" value="<?php echo get_option('forumial_secret'); ?>" readonly="readonly" /></td>
                    </tr>
                </table>
                <?php submit_button(); ?>
            </form>
        </div>
    </div>
    <?php
}

function forumial_sso_validate_token($query) {
    if (!is_array($query) || empty($query['token']) || !is_string($query['token'])) {
        return false;
    }

    $dataString = array();
    foreach ($query as $key => $value) {
        if (in_array($key, array('token'))) continue;
        $dataString[] = $key.'='.$value;
    }
    sort($dataString);

    return hash_hmac('sha256', implode('&', $dataString), get_option('forumial_secret')) == $query['token'];
}

function init_forumial_sso() {
    if (!isset($_GET['forumial']) || !$_GET['forumial'] == 'sso') return;

    require_once 'includes/class-forumial-sso-clients.php';
    $clientsTable = new ForumialSSOClients();
    $client_id = isset($_GET['client_id']) ? sanitize_key($_GET['client_id']) : null;
    $callback = isset($_GET['callback']) ? sanitize_text_field($_GET['callback']) : null;
    $code = isset($_GET['code']) ? sanitize_key($_GET['code']) : null;

    if ( $client_id !== get_option('forumial_clientid') ) {
        header('HTTP/1.1 400 Bad Request');
        header('Content-Type: application/json');
        $response = [
            'error' => 'Client ID is not valid!'
        ];
        wp_send_json($response);
    }

    if (!empty($code)) {
        if ( !forumial_sso_validate_token($_GET) ) {
            header('HTTP/1.1 401 Unauthorized');
            $response = [
                'error' => 'Token is not valid!',
            ];
            wp_send_json($response);
        }
    
        // Check Authen code -> get UserID -> return User Info().
        $client = $clientsTable->getAvailable($code);

        if (empty($client)) {
            header('HTTP/1.1 401 Unauthorized');
            $response = [
                'error' => 'Authorization code is not valid!',
            ];
            wp_send_json($response);
        }

        $user = get_userdata($client->user_id);

        if (!$user) {
            header('HTTP/1.1 404 Not Found');
            $response = [
                'error' => 'User is not found!'
            ];
            wp_send_json($response);
        }

        $response = [
            'email' => $user->user_email,
            'name' => $user->display_name,
            'first_name' => $user->user_firstname,
            'last_name' => $user->user_lastname,
            'username' => $user->user_login,
            'avatar' => get_avatar_url($user->ID),
        ];
        wp_send_json($response);
    }

    if ( ! is_user_logged_in() ) {
		wp_redirect( wp_login_url( $_SERVER['REQUEST_URI'] ) );
		exit;
    }

    $current_user = wp_get_current_user();

    if ($callback) {
        $clientsTable->client_id = $client_id;
        $clientsTable->user_id = $current_user->ID;
        $clientsTable->redirect_uri = $callback;
    
        $client = $clientsTable->save();
    
        // Add one record to Table and return callback with a code. For Example:
        // http://hello.forumial.dev/openid/wordpress?code=000000000000
        $redirect_uri = $callback . '?code='.$client->id;
        wp_redirect($redirect_uri);
        exit;
    }
}

add_action('init', 'init_forumial_sso');
